@@ -23,13 +23,19 @@
 #include "device_p.h"
 #include "device.h"
 #include "adapter.h"
+#include "input.h"
+#include "input_p.h"
+#include "mediaplayer.h"
+#include "mediaplayer_p.h"
 #include "utils.h"
 #include "macros.h"
 
 namespace BluezQt
 {
 
-DevicePrivate::DevicePrivate(const QString &path, const QVariantMap &properties, AdapterPtr adapter)
+static const qint16 INVALID_RSSI = -32768; // qint16 minimum
+
+DevicePrivate::DevicePrivate(const QString &path, const QVariantMap &properties, const AdapterPtr &adapter)
     : QObject()
     , m_dbusProperties(0)
     , m_deviceClass(0)
@@ -38,7 +44,7 @@ DevicePrivate::DevicePrivate(const QString &path, const QVariantMap &properties,
     , m_trusted(false)
     , m_blocked(false)
     , m_legacyPairing(false)
-    , m_rssi(0)
+    , m_rssi(INVALID_RSSI)
     , m_connected(false)
     , m_adapter(adapter)
 {
@@ -71,6 +77,56 @@ void DevicePrivate::init(const QVariantMap &properties)
     m_connected = properties.value(QStringLiteral("Connected")).toBool();
     m_uuids = stringListToUpper(properties.value(QStringLiteral("UUIDs")).toStringList());
     m_modalias = properties.value(QStringLiteral("Modalias")).toString();
+
+    if (!m_rssi) {
+        m_rssi = INVALID_RSSI;
+    }
+}
+
+void DevicePrivate::interfacesAdded(const QString &path, const QVariantMapMap &interfaces)
+{
+    bool changed = false;
+    QVariantMapMap::const_iterator it;
+
+    for (it = interfaces.constBegin(); it != interfaces.constEnd(); ++it) {
+        if (it.key() == Strings::orgBluezInput1()) {
+            m_input = InputPtr(new Input(path, it.value()));
+            m_input->d->q = m_input.toWeakRef();
+            Q_EMIT q.data()->inputChanged(m_input);
+            changed = true;
+        } else if (it.key() == Strings::orgBluezMediaPlayer1()) {
+            m_mediaPlayer = MediaPlayerPtr(new MediaPlayer(path, it.value()));
+            m_mediaPlayer->d->q = m_mediaPlayer.toWeakRef();
+            Q_EMIT q.data()->mediaPlayerChanged(m_mediaPlayer);
+            changed = true;
+        }
+    }
+
+    if (changed) {
+        Q_EMIT q.data()->deviceChanged(q.toStrongRef());
+    }
+}
+
+void DevicePrivate::interfacesRemoved(const QString &path, const QStringList &interfaces)
+{
+    Q_UNUSED(path)
+    bool changed = false;
+
+    Q_FOREACH (const QString &interface, interfaces) {
+        if (interface == Strings::orgBluezInput1()) {
+            m_input.clear();
+            Q_EMIT q.data()->inputChanged(m_input);
+            changed = true;
+        } else if (interface == Strings::orgBluezMediaPlayer1()) {
+            m_mediaPlayer.clear();
+            Q_EMIT q.data()->mediaPlayerChanged(m_mediaPlayer);
+            changed = true;
+        }
+    }
+
+    if (changed) {
+        Q_EMIT q.data()->deviceChanged(q.toStrongRef());
+    }
 }
 
 QDBusPendingReply<> DevicePrivate::setDBusProperty(const QString &name, const QVariant &value)
@@ -114,7 +170,7 @@ void DevicePrivate::propertiesChanged(const QString &interface, const QVariantMa
         } else if (property == QLatin1String("Modalias")) {
             PROPERTY_CHANGED(m_modalias, toString, modaliasChanged);
         } else if (property == QLatin1String("UUIDs")) {
-            uuidsPropertyChanged(stringListToUpper(value.toStringList()));
+            PROPERTY_CHANGED2(m_uuids, stringListToUpper(value.toStringList()), uuidsChanged);
         }
     }
 
@@ -128,11 +184,11 @@ void DevicePrivate::propertiesChanged(const QString &interface, const QVariantMa
         } else if (property == QLatin1String("Icon")) {
             PROPERTY_INVALIDATED(m_icon, QString(), iconChanged);
         } else if (property == QLatin1String("RSSI")) {
-            PROPERTY_INVALIDATED(m_rssi, 0, rssiChanged);
+            PROPERTY_INVALIDATED(m_rssi, INVALID_RSSI, rssiChanged);
         } else if (property == QLatin1String("Modalias")) {
             PROPERTY_INVALIDATED(m_modalias, QString(), modaliasChanged);
         } else if (property == QLatin1String("UUIDs")) {
-            uuidsPropertyChanged(QStringList());
+            PROPERTY_INVALIDATED(m_uuids, QStringList(), uuidsChanged);
         }
     }
 
@@ -162,18 +218,8 @@ void DevicePrivate::classPropertyChanged(quint32 value)
     if (m_deviceClass != value) {
         m_deviceClass = value;
         Q_EMIT q.data()->deviceClassChanged(m_deviceClass);
-        Q_EMIT q.data()->deviceTypeChanged(q.data()->deviceType());
+        Q_EMIT q.data()->typeChanged(q.data()->type());
     }
 }
 
-void DevicePrivate::uuidsPropertyChanged(const QStringList &value)
-{
-    if (m_uuids != value) {
-        m_uuids = value;
-        Q_EMIT q.data()->uuidsChanged(m_uuids);
-    }
-}
-
-#undef PROPERTY_CHANGED
-
 } // namespace BluezQt
