@@ -1,7 +1,7 @@
 /*
  * BluezQt - Asynchronous Bluez wrapper library
  *
- * Copyright (C) 2014 David Rosca <nowrep@gmail.com>
+ * Copyright (C) 2014-2015 David Rosca <nowrep@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -142,12 +142,12 @@ void ManagerPrivate::getManagedObjectsFinished(QDBusPendingCallWatcher *watcher)
         const QString &path = it.key().path();
         const QVariantMapMap &interfaces = it.value();
 
-        if (interfaces.contains(Strings::orgBluezAdapter1())) {
-            addAdapter(path, interfaces.value(Strings::orgBluezAdapter1()));
-        } else if (interfaces.contains(Strings::orgBluezDevice1())) {
-            addDevice(path, interfaces.value(Strings::orgBluezDevice1()));
-        } else if (interfaces.contains(Strings::orgBluezAgentManager1()) && interfaces.contains(Strings::orgBluezProfileManager1())) {
+        interfacesAdded(it.key(), interfaces);
+
+        if (interfaces.contains(Strings::orgBluezAgentManager1())) {
             m_bluezAgentManager = new BluezAgentManager(Strings::orgBluez(), path, DBusConnection::orgBluez(), this);
+        }
+        if (interfaces.contains(Strings::orgBluezProfileManager1())) {
             m_bluezProfileManager = new BluezProfileManager(Strings::orgBluez(), path, DBusConnection::orgBluez(), this);
         }
     }
@@ -157,6 +157,11 @@ void ManagerPrivate::getManagedObjectsFinished(QDBusPendingCallWatcher *watcher)
         return;
     }
 
+    if (!m_bluezProfileManager) {
+        Q_EMIT initError(QStringLiteral("Cannot find org.bluez.ProfileManager1 object!"));
+        return;
+    }
+
     connect(m_dbusObjectManager, &DBusObjectManager::InterfacesAdded,
             this, &ManagerPrivate::interfacesAdded);
     connect(m_dbusObjectManager, &DBusObjectManager::InterfacesRemoved,
@@ -179,10 +184,11 @@ void ManagerPrivate::clear()
     m_loaded = false;
 
     // Delete all devices first
-    Q_FOREACH (DevicePtr device, m_devices.values()) {
+    while (!m_devices.isEmpty()) {
+        DevicePtr device = m_devices.begin().value();
+        m_devices.remove(m_devices.begin().key());
         device->adapter()->d->removeDevice(device);
     }
-    m_devices.clear();
 
     // Delete all adapters
     while (!m_adapters.isEmpty()) {
@@ -221,7 +227,7 @@ AdapterPtr ManagerPrivate::findUsableAdapter() const
 
 void ManagerPrivate::serviceRegistered()
 {
-    qCDebug(BLUEZQT) << "Bluez service registered";
+    qCDebug(BLUEZQT) << "BlueZ service registered";
     m_bluezRunning = true;
 
     load();
@@ -229,7 +235,7 @@ void ManagerPrivate::serviceRegistered()
 
 void ManagerPrivate::serviceUnregistered()
 {
-    qCDebug(BLUEZQT) << "Bluez service unregistered";
+    qCDebug(BLUEZQT) << "BlueZ service unregistered";
 
     bool wasBtOperational = q->isBluetoothOperational();
     m_bluezRunning = false;
@@ -254,6 +260,13 @@ void ManagerPrivate::interfacesAdded(const QDBusObjectPath &objectPath, const QV
             addDevice(path, it.value());
         }
     }
+
+    Q_FOREACH (DevicePtr device, m_devices.values()) {
+        if (path.startsWith(device->ubi())) {
+            device->d->interfacesAdded(path, interfaces);
+            break;
+        }
+    }
 }
 
 void ManagerPrivate::interfacesRemoved(const QDBusObjectPath &objectPath, const QStringList &interfaces)
@@ -267,9 +280,16 @@ void ManagerPrivate::interfacesRemoved(const QDBusObjectPath &objectPath, const
             removeDevice(path);
         }
     }
+
+    Q_FOREACH (DevicePtr device, m_devices.values()) {
+        if (path.startsWith(device->ubi())) {
+            device->d->interfacesRemoved(path, interfaces);
+            break;
+        }
+    }
 }
 
-void ManagerPrivate::adapterRemoved(AdapterPtr adapter)
+void ManagerPrivate::adapterRemoved(const AdapterPtr &adapter)
 {
     disconnect(adapter.data(), &Adapter::poweredChanged, this, &ManagerPrivate::adapterPoweredChanged);
 
@@ -300,10 +320,14 @@ void ManagerPrivate::rfkillStateChanged(Rfkill::State state)
     Q_UNUSED(state)
 
     bool blocked = rfkillBlocked();
+    bool wasBtOperational = q->isBluetoothOperational();
 
     if (m_bluetoothBlocked != blocked) {
         m_bluetoothBlocked = blocked;
         Q_EMIT q->bluetoothBlockedChanged(m_bluetoothBlocked);
+        if (wasBtOperational != q->isBluetoothOperational()) {
+            Q_EMIT q->bluetoothOperationalChanged(q->isBluetoothOperational());
+        }
     }
 }
 
@@ -331,8 +355,8 @@ void ManagerPrivate::addDevice(const QString &devicePath, const QVariantMap &pro
     AdapterPtr adapter = m_adapters.value(properties.value(QStringLiteral("Adapter")).value<QDBusObjectPath>().path());
     Q_ASSERT(adapter);
     DevicePtr device = DevicePtr(new Device(devicePath, properties, adapter));
-    m_devices.insert(devicePath, device);
     device->d->q = device.toWeakRef();
+    m_devices.insert(devicePath, device);
     adapter->d->addDevice(device);
 
     connect(device.data(), &Device::deviceRemoved, q, &Manager::deviceRemoved);
@@ -341,16 +365,25 @@ void ManagerPrivate::addDevice(const QString &devicePath, const QVariantMap &pro
 
 void ManagerPrivate::removeAdapter(const QString &adapterPath)
 {
-    AdapterPtr adapter = m_adapters.take(adapterPath);
+    AdapterPtr adapter = m_adapters.value(adapterPath);
     if (!adapter) {
         return;
     }
 
+    // Make sure we always remove all devices before removing the adapter
+    Q_FOREACH (const DevicePtr &device, adapter->devices()) {
+        removeDevice(device->ubi());
+    }
+
+    m_adapters.remove(adapterPath);
     Q_EMIT adapter->adapterRemoved(adapter);
 
     if (m_adapters.isEmpty()) {
         Q_EMIT q->allAdaptersRemoved();
     }
+
+    disconnect(adapter.data(), &Adapter::adapterChanged, q, &Manager::adapterChanged);
+    disconnect(adapter.data(), &Adapter::poweredChanged, this, &ManagerPrivate::adapterPoweredChanged);
 }
 
 void ManagerPrivate::removeDevice(const QString &devicePath)
@@ -361,9 +394,16 @@ void ManagerPrivate::removeDevice(const QString &devicePath)
     }
 
     device->adapter()->d->removeDevice(device);
+
+    disconnect(device.data(), &Device::deviceChanged, q, &Manager::deviceChanged);
 }
 
-void ManagerPrivate::setUsableAdapter(AdapterPtr adapter)
+bool ManagerPrivate::rfkillBlocked() const
+{
+    return m_rfkill->state() == Rfkill::SoftBlocked || m_rfkill->state() == Rfkill::HardBlocked;
+}
+
+void ManagerPrivate::setUsableAdapter(const AdapterPtr &adapter)
 {
     if (m_usableAdapter == adapter) {
         return;
@@ -379,11 +419,6 @@ void ManagerPrivate::setUsableAdapter(AdapterPtr adapter)
     }
 }
 
-bool ManagerPrivate::rfkillBlocked() const
-{
-    return m_rfkill->state() == Rfkill::SoftBlocked || m_rfkill->state() == Rfkill::HardBlocked;
-}
-
 void ManagerPrivate::dummy()
 {
 }
